Program wait (input,output);

{ Copyright 1988, Jon Radel.  P.O. Box 2276, Reston, VA 22090.

  version 1.0 6Nov88

  May be freely used by all in unmodified form.  All changes to the program
  must be noted in comments in the source code and documentation.  All
  warranties, expressed or implied, are disclaimed.  In no event shall Jon
  Radel be liable for any loss of profit or any other commercial damage,
  including but not limited to special, incidental, consequential or other
  damages.

  WAIT is a general purpose replacement for all the PAUSE and keystroke
  query programs used in BATCH files.  It can wait until a given time,
  wait a given amount of time, wait until any key is hit, or wait until
  a specific answer is typed in.  It will then either run a command or
  simply return, doing nothing.

  Syntax:

         WAIT [UNTIL] hh:mm:ss [command [commandline]]
         WAIT RESPONSE ["message" ["reply"]] [command [commandline]]

  See the documentation for further information.
}


uses Dos, Crt;
{$M 3072,0,0}      { Shrink the stack and get rid of heap entirely }
{$S+}              { Do stack checking, since shrank it }

const
  copyright = ' Copyright 1988 Jon Radel ';
  { types of halts }
  halt_ok = 0;               { normal termination to program }
  halt_noexec = 1;           { user gave response other than "reply" }
  halt_syntax = 2;           { error in entering the parameters }
  halt_break = 3;            { user used ^BREAK or ^C }
  halt_internal = 4;         { internal error, this should never happen }

type
  vartype = (abstime, relative, keystroke); { different syntax variations }

var
  variety : vartype;         { which form of the command is being used }
  sindex,
  hour, minute, second : integer;      { time waiting for }
  chour, cminute, csecond, csec100,    { time right now }
  pindex : word;
  position : byte;                     { position in a string }
  parameter,                           { one or more command line parameters }
  command_todo,                        { name of command to be done }
  command_line : string;               { parameters to pass to command_todo }
  message,                             { message to write on screen for user }
  reply,                               { response required to do command }
  typedin : string[80];                { response that user types in }

procedure writesyntax;
{ This procedure never returns, given the halt at the bottom. }
  begin
    writeln;
    writeln ('WAIT [UNTIL] hh:mm:ss [command [commandline]]');
    writeln ('WAIT RESPONSE ["message" ["reply"]] [command [commandline]]');
    halt(halt_syntax);
    writeln (copyright);     { NOP, but embeds copyright in .EXE file }
  end;

procedure waituntil (hour, minute, second : integer);
{ This procedure does nothing except wait until the time it's handed. }
  begin
    repeat
      { Can break out even if BREAK OFF, it'll eat other keystrokes. }
      if KeyPressed then
        if ReadKey = #3 then
          halt(halt_break);            { This will happen only if BREAK OFF }
      GetTime(chour, cminute, csecond, csec100)
    until (chour=hour) and (cminute=minute) and (csecond=second);
  end;

procedure runcommand (command_todo, command_line : string);
{ This procedure executes the given command, if any, and then exits. }
{ This procedure never returns. }
begin
  if command_todo = '' then
    { There was no command to run, so we're all done now. }
    halt(halt_ok)
  else
    { Actually run the command and *then* exit. }
    begin
      { Get the COMSPEC from environment, and then run COMMAND.COM }
      Exec(GetEnv('COMSPEC'), '/C '+command_todo+command_line);
      { The following variation on the above line doesn't load another
        copy of COMMAND.COM.  If you use it, you have to give the *full*
        path and filename for the file to execute. }
      {Exec(command_todo, command_line);}
      if DosError <> 0 then
        begin
          writeln ('Could not run command. Dos Error: ', DosError);
          writesyntax;
        end;
      halt(DosExitCode);     { return with same exit code as command_todo }
    end;
  halt(halt_internal);       { should never get here }
end;

procedure timeread (var parameter : string;
                    var hour, minute, second : integer);
{ Converts the string in parameter, if a valid time, into integers }
  var
    position : byte;
    errorval : integer;
    paramfrag : string[3];   { fragment of parameter }

  begin
    {$R-}                    { range checking off so we can do our own }
    position := Pos(':', parameter);
    if (position = 0) or (position > 3) then
      begin
        writeln ('Expected time, found: ', parameter);
        writesyntax;
      end;
    paramfrag := copy(parameter, 1, position-1);
    delete(parameter, 1, position);
    val(paramfrag, hour, errorval);
    if (errorval <> 0) or (hour < 0) or (hour > 24) then
      begin
        writeln ('Invalid hour figure: ', paramfrag);
        writesyntax;
      end
    else if (hour = 24) and (variety = abstime) then
      { Allow use of "European" 24:30 instead of 00:30 }
      hour := 0;
    position := Pos(':', parameter);
    if (position = 0) or (position > 3) then
      writesyntax;
    paramfrag := copy(parameter, 1 , position-1);
    delete(parameter, 1, position);
    val(paramfrag, minute, errorval);
    if (errorval <> 0) or (minute < 0) or (minute > 59) then
      begin
        writeln ('Invalid minute figure: ', paramfrag);
        writesyntax;
      end;
    val(parameter, second, errorval);
    if (errorval <> 0) or (second < 0) or (second > 59) then
      begin
        writeln ('Invalid second figure: ', parameter);
        writesyntax;
      end;
  end;

begin
  GetTime(chour, cminute, csecond, csec100);
  command_todo := '';
  command_line := '';
  if ParamCount < 1 then
    writesyntax
  else
    begin
      parameter := ParamStr(1);
      for sindex := 1 to length(parameter) do
        parameter[sindex] := UpCase(parameter[sindex]);
      if parameter = 'UNTIL' then
        variety := abstime
      else if parameter = 'RESPONSE' then
        variety := keystroke
      else { just assume it's a time }
        variety := relative;
    end;
  if variety = abstime then
    begin
      parameter := ParamStr(2);
      timeread (parameter, hour, minute, second);
      if ParamCount >= 3 then
        command_todo := ParamStr(3);
      for pindex := 4 to ParamCount do
        command_line := command_line+' '+ParamStr(pindex);
      waituntil (hour, minute, second);
      runcommand (command_todo, command_line);
    end
  else if variety = relative then
    begin
      timeread (parameter, hour, minute, second);
      second := second + csecond;
      minute := minute + cminute;
      hour := hour + chour;
      if second > 59 then
        begin
          second := second mod 60;
          minute := minute + 1;
        end;
      if minute > 59 then
        begin
          minute := minute mod 60;
          hour := hour + 1;
        end;
      if hour > 23 then
        hour := hour mod 24;
      if ParamCount >= 2 then
        command_todo := ParamStr(2);
      for pindex := 3 to ParamCount do
        command_line := command_line+' '+ParamStr(pindex);
      waituntil (hour, minute, second);
      runcommand (command_todo, command_line);
    end
  else if variety = keystroke then
    begin
      parameter := '';
      message := 'Strike a key when ready . . .';
      reply := '';
      for pindex := 2 to ParamCount do
        parameter := parameter+' '+ParamStr(pindex);
      Delete(parameter, 1, 1);         { Get rid of leading ' ' }
      if parameter[1] = '"' then
        begin
          Delete(Parameter, 1, 1);     { Get rid of leading '"' }
          position := Pos('"', parameter);
          if position = 0 then
            begin
              writeln ('No matching " in message found.');
              writesyntax;
            end
          else
            begin
              message := copy(parameter, 1, position-1);
              if parameter[position+1] = ' ' then
                Delete(Parameter, 1, position+1) { Get rid of message+'" ' }
              else
                { allow for there not being space between messge and reply }
                Delete(parameter, 1, position);  { Get rid of message+'"' }
              if parameter[1] = '"' then
                begin
                  Delete(parameter, 1, 1);       { Get rid of leading '"' }
                  position := Pos('"', parameter);
                  if position = 0 then
                    begin
                      writeln ('No matching " in reply found.');
                      writesyntax;
                    end
                  else
                    begin
                      reply := copy(parameter, 1, position-1);
                      Delete(parameter, 1, position+1);
                    end
                end
            end
        end;
        if parameter <> '' then
          { there is a command following any message and reply }
          begin
            position := Pos(' ', parameter);
            if position <> 0 then
              begin
                command_todo := Copy(Parameter, 1, position-1);
                command_line := Copy(Parameter, position, length(Parameter));
              end
            else
              command_todo := parameter;
          end;
      write (message, ' ');
      if reply <> '' then
        { need specific response }
        begin
          readln (typedin);
          if typedin = reply then
            runcommand(command_todo, command_line)
          else
            halt(halt_noexec);
        end
      else
        { any keystroke will serve, though can ^BREAK or ^C out }
        begin
          if ReadKey <> #3 then
            begin
              writeln;
              runcommand(command_todo, command_line)
            end
          else
            halt(halt_break);
        end;
    end
  else
    begin
      writeln ('Fatal internal error');
      halt(halt_internal);
    end;
end.

